# Iterable:
#   - iter(objet) -> retourne un iterateur
# Iterateur:
#    - next(objet) -> retourne sa valeur suivante


for x in un_iterateur:
    f(x)

for x in un_iterable:
    f(x)

# Équivalent à:

un_iterateur = iter(un_iterable)

while True:
    try:
        f(next(un_iterateur))
    except StopIteration:
        break
    
# Iterables:
# - list
# - tuple
# - set
# - str


# Example
L = [iter(range(10)), iter(range(3))]
for x in L:
    for y in x:
        print(y)

# Affiche 0-9 0-3

for x in L:
    for y in x:
        print(y)
# N'affiche rien, les itérateurs sont vides!

# Applatir un itérateur
## Le transformer en iterable

# - tuple
# - list

# Les décorateur d'itérateurs

# - map
# - tqdm (affiche la progression)

# Quelques opérateurs standard:
# - map (applique une fonction quelconque)
# - enumerate

count = 0
for i in mon_iterateur:
    ...
    count += 1

for count, i in enumerate(mon_iterateur):
    ...

# Exemple:

filter(lambda e:e[0]%2 ==0, enumerate("abcdef"))
## Élimine un élément sur deux de l'itérateur

def une_sur_deux(mot: str):
    return ''.join((map(lambda e:e[1], filter(lambda e:e[0]%2 ==0, enumerate("abcdef")))))

def une_sur_deux_imp(mot: str):
    s = ""
    count = 0
    for a in mot:
        if count % 2 == 0:
            s += a
        count += 1
    return s

# zip: compose deux itérateurs en un seul

# Example

def melange_deux_mot(u: str, v: str) -> str:
    return ''.join(map(lambda e:e[0]+e[1], zip("abcd", "efghi")))

def melange_deux_mot_imp(u: str, v: str) -> str:
    n = min(len(u), len(v))
    s = ""
    for i in range(n):
        s += u[i] + v[i]
    return s


# Les générateurs
from typing import Callable, Iterable, Iterator

def mon_map(f: Callable, it: Iterator) -> Iterator:
    for x in it:
        yield f(x)

def mon_filter(f: Callable, it: Iterator) -> Iterator:
    for x in it:
        if f(x):
            yield x

def mon_zip(it: Iterator, it2: Iterator) -> Iterator:
    while True:
        yield (next(it), next(it2))

def mon_enumerate(it: Iterator) -> Iterator:
    count = 0
    for x in it:
        yield (count, x)
        count += 1


def premiers():
    P = set()
    n = 2
    while True:
        if not any(map(lambda e:n%e == 0, P)):
            P.add(n)
            yield n
        n += 1

it = filter(lambda e:e[1]-e[0]==2, zip(P, P2))

# it va énumérer les nombres premiers jumeaux !
